@@ -12,28 +12,46 @@
 #
 #-------------------------------------------------------------------------------
 
-from CommandManager import *
 import xml.dom.minidom as xml
+import os
 import pygst
 pygst.require("0.10")
 import gst
-from Monitored import *
-from Utils import *
-import os, Globals
+from Monitored import Monitored
+import Utils
+import UndoSystem
+import Globals
+import gettext
+import urllib
+_ = gettext.gettext
 
 #=========================================================================
 
-class Event(Monitored, CommandManaged):
-	""" This class handles maintaing the information for a single audio 
-		event. This is normally a fragment of a recorded file.
+class Event(Monitored):
+	"""
+	This class handles maintaing the information for a single audio 
+	event, normally, a fragment of a recorded file.
 	"""
 	
-	#state changed types (to be sent through the Monitored class)
-	WAVEFORM, MOVE, LENGTH, CORRUPT = range(4)
+	""" State changed types (to be sent through the Monitored class) """
+	WAVEFORM, MOVE, LENGTH, CORRUPT, LOADING = range(5)
+	
+	""" The level sample interval in seconds """
+	LEVEL_INTERVAL = 0.01
 	
 	#_____________________________________________________________________
 	
 	def __init__(self, instrument, file=None, id=None, filelabel=None):
+		"""
+		Creates a new instance of Event.
+		
+		Parameters:
+			instrument -- Instrument associated with this Event.
+			file -- the file this Event should play.
+			id -- unique ID for this Event. If it's taken, a new one will be generated.
+			filelabel -- label to print in error messages.
+						It can be different	from the file parameter.
+		"""
 		Monitored.__init__(self)
 		
 		self.start = 0.0			# Time in seconds at which the event begins
@@ -45,7 +63,7 @@ class Event(Monitored, CommandManaged):
 
 		# the label is the filename to print in error messages
 		# if it differs from the real filename (i.e its been copied into the project)
-		if filelabel!=None:
+		if filelabel != None:
 			self.filelabel = filelabel
 		else:
 			self.filelabel = file
@@ -53,7 +71,7 @@ class Event(Monitored, CommandManaged):
 		self.isSelected = False		# True if the event is currently selected
 		self.name = "New Event"		# Name of this event
 		
-		self.selection  = [0, 0]			# List start and end of selection (for fades, etc) measured in seconds 
+		self.selection  = [0, 0]	# List start and end of selection (for fades, etc) measured in seconds 
 		self.levels = []			# Array of audio levels to be drawn for this event
 		
 		self.id = instrument.project.GenerateUniqueID(id)  #check is id is already taken, then set it.
@@ -62,8 +80,12 @@ class Event(Monitored, CommandManaged):
 		
 		self.offset = 0.0			# Offset through the file in seconds
 		self.isLoading = False		# True if the event is currently loading level data
+		self.isDownloading = False	# True if the event is currently loading from a remote source.
+		self.isRecording = False		# True if the event is currently loading level data from a live recording
 		self.loadingLength = 0 		# The length of the file in seconds as its being rendered
 		self.lastEnd = 0 			# The last length of the loading file - used to minimise redraws
+		self.loadingPipeline = None	# The Gstreamer pipeline used to load the waveform
+		self.bus = None			# The bus to monitor messages on the loadingPipeline
 		
 		self.CreateFilesource()
 
@@ -84,6 +106,11 @@ class Event(Monitored, CommandManaged):
 	#_____________________________________________________________________
 	
 	def CreateFilesource(self):	
+		"""
+		Creates a new GStreamer file source with an unique id.
+		It then calls SetProperties() to populate the new object's
+		properties.
+		"""
 		Globals.debug("create file source")
 		if not self.filesrc:
 			self.filesrc = gst.element_factory_make("gnlfilesource", "Event_%d"%self.id)
@@ -95,37 +122,35 @@ class Event(Monitored, CommandManaged):
 	#_____________________________________________________________________
 		
 	def SetProperties(self):
+		"""
+		Sets basic Event properties like location, start, duration, etc.
+		"""
 		if self.file:
-			Globals.debug("setting event properties")
-			self.filesrc.set_property("location", self.file)
-			self.filesrc.set_property("start", long(self.start * gst.SECOND))
-			self.filesrc.set_property("duration", long(self.duration * gst.SECOND))
-			self.filesrc.set_property("media-start", long(self.offset * gst.SECOND))
-			self.filesrc.set_property("media-duration", long(self.duration * gst.SECOND))
-			self.filesrc.set_property("priority", 2)
-			
-			Globals.debug("location = " + self.file)
-			Globals.debug("start = " + str(long(self.start * gst.SECOND)))
-			Globals.debug("duration = " + str(long(self.duration * gst.SECOND)))
-			Globals.debug("media-start = " + str(long(self.offset * gst.SECOND)))
-			Globals.debug("media-duration = " + str(long(self.duration * gst.SECOND)))
-			
-			Globals.debug("event properties set")
+			Globals.debug("setting event properties:")
+			propsDict = {"location" : self.file,
+					"start" : long(self.start * gst.SECOND),
+					"duration" : long(self.duration * gst.SECOND),
+					"media-start" : long(self.offset * gst.SECOND),
+					"media-duration" : long(self.duration * gst.SECOND),
+					"priority" : 2
+					}
+					
+			for prop, value in propsDict.iteritems():
+				self.filesrc.set_property(prop, value)
+				Globals.debug("\t", prop, "=", value)
 
 	#_____________________________________________________________________
 
 	def StoreToXML(self, doc, parent, graveyard=False):
-		""" Converts this Event into an XML representation suitable for 
-			saving to a file.
+		"""
+		Converts this Event into an XML representation suitable for saving to a file.
 
-			doc
-				The XML dcxument object we're saving to.
-			parent
-				The parent node that the serialised Event should be added
-				to.
-			graveyard
-				True if this Event is on the graveyard stack, and should
-				be serialised as a dead event.
+		Parameters:
+			doc -- the XML document object the Event will be saved to.
+			parent -- the parent node that the serialized Event should
+						be added to.
+			graveyard -- True if this Event is on the graveyard stack,
+						and should be serialized as a dead Event.
 		"""
 		if graveyard:
 			ev = doc.createElement("DeadEvent")
@@ -138,7 +163,7 @@ class Event(Monitored, CommandManaged):
 		ev.appendChild(params)
 		
 		items = ["start", "duration", "isSelected", 
-				  "name", "offset", "file", "isLoading"
+				  "name", "offset", "file", "isLoading", "isRecording"
 				]
 				
 		#Since we are saving the path to the project file, don't delete it on exit
@@ -150,7 +175,7 @@ class Event(Monitored, CommandManaged):
 			# If the file is in the audio dir, just include the filename, not the absolute path
 			self.file = os.path.basename(self.file)
 		
-		StoreParametersToXML(self, doc, params, items)
+		Utils.StoreParametersToXML(self, doc, params, items)
 		
 		# Put self.file back to its absolute path
 		self.file = self.temp
@@ -158,61 +183,36 @@ class Event(Monitored, CommandManaged):
 		
 		xmlPoints = doc.createElement("FadePoints")
 		ev.appendChild(xmlPoints)
-		StoreDictionaryToXML(doc, xmlPoints, self.__fadePointsDict, "FadePoint")
+		Utils.StoreDictionaryToXML(doc, xmlPoints, self.__fadePointsDict, "FadePoint")
 		
 		if self.levels:
 			levelsXML = doc.createElement("Levels")
 			ev.appendChild(levelsXML)
 			stringList = map(str, self.levels)
 			levelsXML.setAttribute("value", ",".join(stringList))
-			
-	#_____________________________________________________________________
-			
-	def LoadFromXML(self, node):
-		""" Restores an event from its serialised XML representation.
-
-			node
-				The XML node to retreive data from.
-		"""
-		params = node.getElementsByTagName("Parameters")[0]
-		
-		LoadParametersFromXML(self, params)
-		
-		if not os.path.isabs(self.file):
-			# If there is a relative path for self.file, assume it is in the audio dir
-			self.file = os.path.join(self.instrument.path, self.file)
-		
-		try:
-			xmlPoints = node.getElementsByTagName("FadePoints")[0]
-		except IndexError:
-			Globals.debug("Missing FadePoints in Event XML")
-		else:
-			self.__fadePointsDict = LoadDictionaryFromXML(xmlPoints)
-		
-		try:	
-			levelsXML = node.getElementsByTagName("Levels")[0]
-		except IndexError:
-			Globals.debug("No event levels in project file")
-			self.GenerateWaveform()
-		else: 
-			if levelsXML.nodeType == xml.Node.ELEMENT_NODE:
-				value = str(levelsXML.getAttribute("value"))
-				self.levels = map(float, value.split(","))
-
-		if self.isLoading == True:
-			self.GenerateWaveform()
-
-		self.__UpdateAudioFadePoints()
-		self.CreateFilesource()
 		
 	#_____________________________________________________________________
 		
 	def __repr__(self):
+		"""
+		Creates a representation string of the Event.
+
+		Returns:
+			a string representing the name, id, start and duration
+			for this Event.
+		"""
 		return "Event: '%s' (%d) : %d -> %d" % (self.name, self.id, self.start, self.duration)
 		
 	#_____________________________________________________________________
 	
 	def __cmp__(self, object):
+		"""
+		Compares two Events for equality.
+		
+		Returns:
+			True -- the Events are equal.
+			False -- the Events are different.
+		"""
 		if type(object) != type(self):
 			return -1
 		else:
@@ -222,16 +222,15 @@ class Event(Monitored, CommandManaged):
 	
 	#_____________________________________________________________________
 	
+	@UndoSystem.UndoCommand("Move", "start", "temp")
 	def Move(self, frm, to):
-		'''	Moves this Event.
+		"""
+		Moves this Event in time.
 
-			frm
-				The time we're moving from.
-			to
-				The time we're moving to.
-
-			undo : Move: start, temp
-		'''
+		Parameters:
+			frm -- the time the Event's moving from.
+			to -- the time the Event's moving to.
+		"""
 		self.temp = frm
 		self.start = to
 		self.SetProperties()
@@ -239,72 +238,50 @@ class Event(Monitored, CommandManaged):
 	#_____________________________________________________________________
 	
 	def Split(self, split_point, id=-1):
-		'''
-			Splits this event at time offset split_point in seconds. If
-			id is specified, then the created event will be pulled from
-			the graveyard (for undo/redo compatibility). Returns the
-			newly created event, which is the one on the right (after the splitpoint).
-		
-			undo : Join: temp
-		'''		
-		if id == -1:
-			e = self.split_event(split_point)
-			e.SetProperties()
-			self.instrument.events.append(e)
-			self.temp = e.id
-			self.SetProperties()
-			self.StateChanged(self.LENGTH)
-			
-			return e
-		else:
-			d = self.duration
-			self.duration = split_point
-			
-			event = [x for x in self.instrument.graveyard if x.id == id][0]
-			self.instrument.events.append(event)
-			
-			nl = int(len(self.levels) * (split_point / d))
-			self.levels = self.levels[:nl]
-			
-			self.temp = event.id
-			
-			self.StateChanged(self.LENGTH)
-		
+		"""
+		Dummy function kept for compatibility with 0.2 project files.
+		Parameters are the same as SplitEvent().
+		"""
+		self.SplitEvent(split_point)
+	
 	#_____________________________________________________________________
 	
 	def Join(self, joinEventID):
-		''' Joins 2 events together. 
-
-			joinEventID
-				The ID of the Event to join to this one.
-
-			undo : Split: temp, temp2
-		'''
-		event = [x for x in self.instrument.events if x.id == joinEventID][0]
-
-		self.temp = self.duration
-		self.temp2 = event.id
-		
-		self.join_event(event)
-		# Now that they're joined, move rightEvent to the graveyard
-		self.instrument.events.remove(event)
-		self.instrument.graveyard.append(event)
-		
-		self.StateChanged(self.LENGTH)
-
+		"""
+		Dummy function kept for compatibility with 0.2 project files.
+		Parameters are the same as JoinEvent().
+		"""
+		self.JoinEvent(joinEventID)
+	
 	#_____________________________________________________________________
 	
-	def split_event(self, split_point, cutRightSide=True):
-		"""helper function for Split() and Trim(). All other methods and classes
-		   should not invoke this function directly since there is no undo for it.
-		   
-		   If cutRightSide is True, a new event will be created to represent 
-		   the piece on the right which was split. This instance will be the one on the left.
-		   If cutRightSide is False, this instance is the one on the right.
+	@UndoSystem.UndoCommand("JoinEvent", "temp", "temp2")
+	def SplitEvent(self, split_point, cutRightSide=True, eventID=-1):
+		"""
+		Splits this Event.
+		
+		Parameters:
+			split_point -- time offset split_point in seconds to start the split.
+			cutRightSide -- if True, a new event will be created to represent
+					the piece on the right which was split. This instance
+					will be the one on the left.
+					if False, this instance is the one on the right.
+			eventID -- the ID of an event that was previously used for the same
+					split. This should only be used by the undo stack and will
+					help prevent the creation of multiple garbage copies of the
+					same event each time a split is undone and redone.
+					
+		Returns:
+			the newly created event.
 		"""
 		dur = self.duration
 		
-		e = Event(self.instrument, self.file)
+		#if we were given an event ID, reuse that event instead of creating a new one
+		if eventID >= 0:
+			e = [x for x in self.instrument.graveyard if x.id == eventID][0]
+			self.instrument.graveyard.remove(e)
+		else:
+			e = Event(self.instrument, self.file)
 		e.name = self.name
 		
 		dictLeft = {}
@@ -331,8 +308,6 @@ class Event(Monitored, CommandManaged):
 			
 			self.__fadePointsDict = dictLeft
 			e.__fadePointsDict = dictRight
-			self.__UpdateAudioFadePoints()
-			e.__UpdateAudioFadePoints()
 		else:
 			e.start = self.start
 			e.offset = self.offset
@@ -348,21 +323,39 @@ class Event(Monitored, CommandManaged):
 			
 			self.__fadePointsDict = dictRight
 			e.__fadePointsDict = dictLeft
-			self.__UpdateAudioFadePoints()
-			e.__UpdateAudioFadePoints()
+			
+		self.__UpdateAudioFadePoints()
+		e.__UpdateAudioFadePoints()
+		e.SetProperties()
+		self.instrument.events.append(e)
+		self.SetProperties()
+		self.StateChanged(self.LENGTH)
+		self.StateChanged(self.MOVE)
+		
+		#undo parameters
+		self.temp = e.id
+		self.temp2 = cutRightSide
 		
 		return e
-	
+		
 	#_____________________________________________________________________
 	
-	def join_event(self, joinEvent, joinToRight=True):
-		"""helper function for Join() and Trim(). All other methods and classes
-		   should not invoke this function directly since there is no undo for it.
-		   
-		   After joining the events on either side, this method will not remove the event
-		   from the instrument lane. This must be done from the function that called this one.
+	@UndoSystem.UndoCommand("SplitEvent", "temp", "temp2", "temp3")
+	def JoinEvent(self, joinEvent, joinToRight=True):
 		"""
+		Joins two Events together.
+		
+		Parameters:
+			joinEvent -- the ID of the Event, or the Event object to join to this one.
+			joinToRight -- True if the event will be joined on the right side
+		"""
+		eventObjectList = [x for x in self.instrument.events if x.id == joinEvent]
+		if eventObjectList:
+			joinEvent = eventObjectList[0]
+		
 		if joinToRight:
+			self.temp = self.duration
+			
 			for key, value in joinEvent.__fadePointsDict.iteritems():
 				self.__fadePointsDict[key + self.duration] = value
 			#remove the point on either edge that was created when they were split
@@ -374,6 +367,8 @@ class Event(Monitored, CommandManaged):
 			#update the fade point list after the level, and duration because it depends on them
 			self.__UpdateAudioFadePoints()
 		else:
+			self.temp = joinEvent.duration
+		
 			self.start = joinEvent.start
 			self.offset = joinEvent.offset
 			self.duration += joinEvent.duration
@@ -387,33 +382,49 @@ class Event(Monitored, CommandManaged):
 				del newDict[joinEvent.duration]
 			self.__fadePointsDict = newDict
 			self.__UpdateAudioFadePoints()
+			
+		# Now that they're joined, move delete the rightEvent
+		if joinEvent in self.instrument.events:
+			self.instrument.events.remove(joinEvent)
+		if not joinEvent in self.instrument.graveyard:
+			self.instrument.graveyard.append(joinEvent)
 		
+		self.StateChanged(self.LENGTH)
+		self.StateChanged(self.MOVE)
+		
+		self.temp2 = joinToRight
+		self.temp3 = joinEvent.id
+
 	#_____________________________________________________________________
 	
 	def Trim(self, start_split, end_split):
-		""" Splits the event at points start_split and end_split
-		    and then deletes the first and last sections leaving only
-		    the middle section.
-
-		    start_split
-		   		The time for the start of the trim
-			end_split
-				The time for the end of the trim
-		   
-		    undo : UndoTrim: temp, temp2
 		"""
-		# Split off the left section of the event, then put it in the graveyard for undo
-		leftSplit = self.split_event(start_split, False)
-		self.instrument.graveyard.append(leftSplit)
-		self.temp = leftSplit.id
+		Splits the Event and then deletes the first and last sections,
+		leaving only the middle section.
+		
+		Parameters:
+			start_split -- the time for the start of the trim.
+			end_split -- the time for the end of the trim.
+		"""
+		
+		if start_split > end_split or (start_split <= 0 and end_split >= self.duration):
+			#both points must not be right at the edges, or there is nothing to split
+			return
+			
+		undoAction = self.instrument.project.NewAtomicUndoAction()
+
+		if 0 < start_split < self.duration:
+			# Split off the left section of the event
+			leftSplit = self.SplitEvent(start_split, False, _undoAction_=undoAction)
+			self.instrument.DeleteEvent(leftSplit.id, _undoAction_=undoAction)
 		
 		#Adjust the end_split value since splitting the left has changed self.duration
 		end_split = end_split - start_split
 		
-		# Split off the right section of the event, then put it in the graveyard for undo
-		rightSplit = self.split_event(end_split)
-		self.instrument.graveyard.append(rightSplit)
-		self.temp2 = rightSplit.id
+		if 0 < end_split < self.duration:
+			# Split off the right section of the event
+			rightSplit = self.SplitEvent(end_split, _undoAction_=undoAction)
+			self.instrument.DeleteEvent(rightSplit.id, _undoAction_=undoAction)
 		
 		self.SetProperties()
 		self.StateChanged(self.LENGTH)
@@ -421,67 +432,80 @@ class Event(Monitored, CommandManaged):
 	#_____________________________________________________________________
 	
 	def UndoTrim(self, leftID, rightID):
-		"""Resurrects two pieces from the graveyard and joins them to
-		   either side of this event.
-		   
-		   undo : Trim: temp, temp2
 		"""
-		leftEvent = [x for x in self.instrument.graveyard if x.id == leftID][0]
-		rightEvent = [x for x in self.instrument.graveyard if x.id == rightID][0]
+		Resurrects two pieces from the graveyard and joins them to
+		either side of this Event.
 		
-		self.temp = leftEvent.duration
-		self.temp2 = leftEvent.duration + self.duration
+		Considerations: 
+			This method is kept here for undo compatibility with Jokosher
+			version 0.2 project files, and is never called when a 0.9 or
+			greater project is being used.
 		
-		self.join_event(leftEvent, False)
-		self.join_event(rightEvent)
+		Parameters:
+			leftID -- id of the left Event to be resurrected.
+			rightID -- id of the right Event to be resurrected.
+		"""
+		undoAction = self.instrument.project.NewAtomicUndoAction()
 		
-		self.instrument.graveyard.remove(leftEvent)
-		self.instrument.graveyard.remove(rightEvent)
-		
-		self.SetProperties()
-		self.StateChanged(self.LENGTH)
+		self.instrument.ResurrectEvent(leftID, _undoAction_=undoAction)
+		self.JoinEvent(leftID, False, _undoAction_=undoAction)
+		self.instrument.ResurrectEvent(rightID, _undoAction_=undoAction)
+		self.JoinEvent(rightID, _undoAction_=undoAction)
 		
 	#_____________________________________________________________________
 	
-	def Delete(self):
-		"""	Deletes this Event and sends it to the graveyard to reflect
-			on what it has done.
-
-			undo : Resurrect
+	def Delete(self, _undoAction_=None):
 		"""
-		self.instrument.graveyard.append(self)
-		self.instrument.events.remove(self)
-		self.instrument.composition.remove(self.filesrc)
-
+		Deletes this Event and sends it to the graveyard.
+		"""
+		if _undoAction_:
+			self.instrument.DeleteEvent(self.id, _undoAction_=_undoAction_)
+		else:
+			self.instrument.DeleteEvent(self.id)
+			
 	#_____________________________________________________________________
 	
-	def Resurrect(self):
-		""" Brings this Event back from the graveyard.
-
-			undo : Delete
+	def Resurrect(self, _undoAction_=None):
 		"""
-		self.instrument.events.append(self)
-		self.instrument.graveyard.remove(self)
-		self.CreateFilesource()
+		Brings this event back from the graveyard.
 		
-	#______________________________________________________________________
-				
+		Considerations:
+			This method is made obsolete by instrument.ResurrectEvent(),
+			but is still kept here for 0.2 undo history compatibility.
+		"""
+		if _undoAction_:
+			self.instrument.ResurrectEvent(self.id, _undoAction_=_undoAction_)
+		else:
+			self.instrument.ResurrectEvent(self.id)
+			
+	#_____________________________________________________________________
+	
 	def bus_message(self, bus, message):
-		""" Handler for the GStreamer bus messages relevant to this Event. 
-			At the moment this is used to report on how the loading 
-			progress is going.
 		"""
-
+		Handler for the GStreamer bus messages relevant to this Event.
+		At the moment, this is used to report on how the loading progress
+		is going.
+		
+		Parameters:
+			bus -- GStreamer bus sending the message.
+			message -- GStreamer message.
+			
+		Returns:
+			True -- the Event is loading.
+			False -- the Event isn't loading.
+		"""
 		if not self.isLoading:
 			return False
 
 		st = message.structure
 		if st:
 			if st.get_name() == "level":
+				newLevel = self.__CalculateAudioLevel(st["peak"])
+				self.levels.append(newLevel)
+				
 				end = st["endtime"] / float(gst.SECOND)
-				self.levels.append(DbToFloat(st["peak"][0]))
 				self.loadingLength = int(end)
-											
+				
 				# Only send events every second processed to reduce GUI load
 				if self.loadingLength != self.lastEnd:
 					self.lastEnd = self.loadingLength 
@@ -491,25 +515,39 @@ class Event(Monitored, CommandManaged):
 	#_____________________________________________________________________
 		
 	def bus_eos(self, bus, message):	
-		""" Handler for the GStreamer End Of Stream message. Currently
-			used when the file is loading and is being rendered. This
-			function is called at the end of the file loading process and
-			finalises the rendering.
+		"""
+		Handler for the GStreamer End Of Stream message. Currently
+		used when the file is loading and is being rendered. This
+		function is called at the end of the file loading process and
+		finalises the rendering.
+		
+		Parameters:
+			bus -- GStreamer bus sending the message.
+			message -- GStreamer message.
+			
+		Returns:
+			False -- stops the signal propagation. *CHECK*
 		"""
 		if message.type == gst.MESSAGE_EOS:
 			
 			# Update levels for partial events
-			q = self.bin.query_duration(gst.FORMAT_TIME)
-			length = q[0] / float(gst.SECOND)
+			q = self.loadingPipeline.query_duration(gst.FORMAT_TIME)
+			length = float(q[0] / float(gst.SECOND))
 			
-			if self.offset > 0 or self.duration != length:
+			#we're at EOS, and still have no value for duration
+			if not self.duration:
+				if length:
+					self.duration = length
+				else:
+					self.duration = self.loadingLength
+			
+			if length and (self.offset > 0 or self.duration != length):
 				dt = int(self.duration * len(self.levels) / length)
 				start = int(self.offset * len(self.levels) / length)
 				self.levels = self.levels[start:start+dt]
 			
 			# We're done with the bin so release it
-			self.bin.set_state(gst.STATE_NULL)
-			self.isLoading = False
+			self.StopGenerateWaveform()
 			
 			# Signal to interested objects that we've changed
 			self.StateChanged(self.WAVEFORM)
@@ -518,13 +556,19 @@ class Event(Monitored, CommandManaged):
 	#_____________________________________________________________________
 
 	def bus_message_statechange(self, bus, message):
-		""" Handler for the GStreamer statechange message.
+		"""
+		Handler for the GStreamer statechange message.
+		
+		Parameters:
+			bus -- GStreamer bus sending the message.
+			message -- GStreamer message.
 		"""
 		# state has changed
 		try:
-			q = self.bin.query_duration(gst.FORMAT_TIME)
-			if self.duration == 0:
-				self.duration = float(q[0] / float(gst.SECOND))
+			time = self.loadingPipeline.query_duration(gst.FORMAT_TIME)
+			if self.duration == 0 and time[0] != 0:
+				self.duration = float(time[0] / float(gst.SECOND))
+				
 				#update position with proper duration
 				self.MoveButDoNotOverlap(self.start)
 				self.SetProperties()
@@ -537,39 +581,202 @@ class Event(Monitored, CommandManaged):
 	#_____________________________________________________________________
 
 	def bus_error(self, bus, message):
-		""" Handler for when things go completely wrong with GStreamer.
 		"""
-		Globals.debug("bus error")
-		self.StateChanged(self.CORRUPT)
+		Handler for when things go completely wrong with GStreamer.
+		
+		Parameters:
+			bus -- GStreamer bus sending the message.
+			message -- GStreamer message.
+		"""
+		error, debug = message.parse_error()
+		
+		Globals.debug("Event bus error:", str(error), str(debug))
+		self.StateChanged(self.CORRUPT, str(error), str(debug))
+	
+	#_____________________________________________________________________
+	
+	def bus_message_tags(self, bus, message):
+		"""
+		Handler for catching audio file tags that Gstreamer throws.
+		
+		Parameters:
+			bus -- GStreamer bus sending the message.
+			message -- GStreamer message.
+		"""
+		Globals.debug("recieved group of tags")
+		st = message.structure
+		
+		title, artist = None, None
+		if st.has_key("title"):
+			title = st["title"]
+		if st.has_key("artist"):
+			artist = st["artist"]
+		
+		#check which tags actually contain something
+		if title and artist:
+			self.name = _("%(title)s by %(artist)s") % {"title":title, "artist":artist}
+		elif title:
+			self.name = title
 	
 	#_____________________________________________________________________
 	
 	def GenerateWaveform(self):
-		""" Renders the level information for the GUI.
 		"""
-		pipe = """filesrc name=src location=%s ! decodebin ! audioconvert ! 
-		level interval=100000000 message=true ! fakesink""" % self.file.replace(" ", "\ ")
-		self.bin = gst.parse_launch(pipe)
+		Renders the level information for the GUI.
+		"""
+		pipe = """filesrc name=src location=%s ! decodebin ! audioconvert ! level interval=%d message=true ! fakesink""" 
+		pipe = pipe % (self.file.replace(" ", "\ "), self.LEVEL_INTERVAL * gst.SECOND)
+		self.loadingPipeline = gst.parse_launch(pipe)
 
-		self.bus = self.bin.get_bus()
+		self.bus = self.loadingPipeline.get_bus()
 		self.bus.add_signal_watch()
 		self.bus.connect("message::element", self.bus_message)
+		self.bus.connect("message::tag", self.bus_message_tags)
 		self.bus.connect("message::state-changed", self.bus_message_statechange)
 		self.bus.connect("message::eos", self.bus_eos)
 		self.bus.connect("message::error", self.bus_error)
 
 		self.levels = []
 		self.isLoading = True
+		self.StateChanged(self.LOADING)
 
-		self.bin.set_state(gst.STATE_PLAYING)
+		self.loadingPipeline.set_state(gst.STATE_PLAYING)
 
 	#_____________________________________________________________________
+	
+	def CopyAndGenerateWaveform(self, uri):
+		"""
+		Copies the audio file to the new file location and reads the levels
+		at the same time.
+		"""
+		if not gst.element_make_from_uri(gst.URI_SRC, uri):
+			#This means that here is no gstreamer src element on the system that can handle this URI type.
+			return False
+		
+		pipe = """%s ! tee name=mytee mytee. ! queue ! filesink location=%s """ +\
+		"""mytee. ! queue ! decodebin ! audioconvert ! level interval=%d message=true ! fakesink""" 
+		pipe = pipe % (urllib.quote(uri,":/"), self.file.replace(" ", "\ "), self.LEVEL_INTERVAL * gst.SECOND)
+		self.loadingPipeline = gst.parse_launch(pipe)
 
+		self.bus = self.loadingPipeline.get_bus()
+		self.bus.add_signal_watch()
+		self.bus.connect("message::element", self.bus_message)
+		self.bus.connect("message::tag", self.bus_message_tags)
+		self.bus.connect("message::state-changed", self.bus_message_statechange)
+		self.bus.connect("message::eos", self.bus_eos)
+		self.bus.connect("message::error", self.bus_error)
+
+		self.levels = []
+		self.isLoading = True
+		self.StateChanged(self.LOADING)
+
+		self.loadingPipeline.set_state(gst.STATE_PLAYING)
+		
+		return True
+		
+	#_____________________________________________________________________
+	
+	def StopGenerateWaveform(self, finishedLoading=True):
+		"""
+		Stops the internal pipeline that loads the waveform from this event's file.
+		
+		Parameters:
+			finishedLoading -- True if the event has finished loading the waveform,
+					False if the loading is being cancelled.
+		"""
+		if self.bus:
+			self.bus.remove_signal_watch()
+			self.bus = None
+		if self.loadingPipeline:
+			self.loadingPipeline.set_state(gst.STATE_NULL)
+			
+			if self.isDownloading:
+				# If we are currently downloading, we can't restart later, 
+				# so cancel regardless of the finishedLoading boolean's value.
+				self.isDownloading = False
+				self.isLoading = False
+			else:
+				self.isLoading = not finishedLoading
+			
+			self.loadingPipeline = None
+			self.loadingLength = 0
+			self.StateChanged(self.LOADING)
+	
+	#_____________________________________________________________________
+
+	def recording_bus_level(self, bus, message):
+		"""
+		Handler for the GStreamer bus messages relevant to this Event.
+		At the moment this is used to report on how the recording
+		progress is going. *CHECK*
+		
+		Parameters:
+			bus -- GStreamer bus sending the message.
+			message -- GStreamer message.
+			
+		Returns:
+			True -- the Event is recording.
+			False -- the Event isn't recording.
+		"""
+		if not self.isRecording:
+			return False
+		
+		st = message.structure
+		if st and message.src.get_name() == "recordlevel":
+			newLevel = self.__CalculateAudioLevel(st["peak"])
+			self.levels.append(newLevel)
+			
+			end = st["endtime"] / float(gst.SECOND)
+			#Round to one decimal place so it updates 10 times per second
+			self.loadingLength = round(end, 1)
+			
+			# Only send events every second processed to reduce GUI load
+			if self.loadingLength != self.lastEnd:
+				self.lastEnd = self.loadingLength 
+				self.StateChanged(self.LENGTH) # tell the GUI
+		return True
+		
+	#_____________________________________________________________________
+	
+	def __CalculateAudioLevel(self, channelLevels):
+		"""
+		Calculates an average for all channel levels.
+		
+		Parameters:
+			channelLevels -- list of levels from each channel.
+			
+		Returns:
+			an average level, also taking into account negative infinity numbers,
+			which will be discarded in the average.
+		"""
+		negInf = float("-inf")
+		peaktotal = 0
+		peakcount = 0
+		for peak in channelLevels:
+			#don't add -inf values cause 500 + -inf is still -inf
+			if peak != negInf:
+				peaktotal += peak
+				peakcount += 1
+		#avoid a divide by zero here
+		if peakcount > 0:
+			peaktotal /= peakcount
+		#it must be put back to -inf if nothing has been added to it, so that the DbToFloat conversion will work
+		elif peakcount == 0:
+			peaktotal = negInf
+		
+		#convert to 0...1 float, and return
+		return Utils.DbToFloat(peaktotal)
+		
+	#_____________________________________________________________________
+	
 	def SetSelected(self, sel):
-		""" Enables or disables the selection state for this event.
+		"""
+		Enables or disables the selection state for this Event.
 
-			sel
-				The new selection state (Should be True or False).
+		Parameters:
+			sel -- the new selection state:
+					True = the Event has been selected.
+					False = the Event has been deselected.
 		"""
 		# No need to call StateChanged when there is no change in selection state
 		if self.isSelected is not sel:
@@ -579,13 +786,15 @@ class Event(Monitored, CommandManaged):
 	#_____________________________________________________________________
 	
 	def MayPlace(self, xpos):
-		""" Checks if this event could be placed at xpos without 
-			overlapping another Event on the same Instrument.
+		"""
+		Checks if this event could be placed at xpos without 
+		overlapping another Event on the same Instrument.
 
-			xpos
-				The potential start position to check
-			Returns
-				True if it's OK to place the Event at xpos, false if not.
+		Parameters:
+			xpos -- the potential start position to check.
+		
+		Returns:
+			True if it's OK to place the Event at xpos, False if not.
 		"""
 		for e in self.instrument.events:
 			if e is self:
@@ -597,9 +806,13 @@ class Event(Monitored, CommandManaged):
 	#_____________________________________________________________________
 	
 	def MoveButDoNotOverlap(self, xpos):
-		"""This method will attempt to move this event to the given position (xpos)
-		   If the position requires overlapping, this event will be put flush
-		   against the closest side of the event which is in the way.
+		"""
+		This method will attempt to move this Event to the given position.
+		If the position requires overlapping, this Event will be put flush
+		against the closest side of the Event which is in the way.
+
+		Parameters:
+			xpos -- the potential position to move the Event to.
 		"""
 		alreadyTriedRemovingOverlap = False
 		self.instrument.events.sort()
@@ -639,13 +852,17 @@ class Event(Monitored, CommandManaged):
 	
 	#_____________________________________________________________________
 	
+	@UndoSystem.UndoCommand("RemoveAudioFadePoints", "temp", "temp2", "temp3", "temp4")
 	def AddAudioFadePoints(self, firstPoint, secondPoint, firstVolume, secondVolume):
 		"""
-		   Add the two passed points to the audioFadePoints list.
-		   If either point exists already, replace it, and resort
-		   the list by time.
-		   
-		   undo : RemoveAudioFadePoints: temp, temp2, temp3, temp4
+		Adds two fade points to the audioFadePoints list.
+		If either point exists already, replaces it, and resorts the list by time.
+		
+		Parameters:
+			firstPoint -- point in time, where the fade starts.
+			secondPoint -- point in time, where the fade ends.
+			firstVolume -- value of the initial fade volume.
+			secondVolume -- value of the final fade volume.
 		"""
 		#for command manager to use with undo
 		self.temp = firstPoint
@@ -668,14 +885,20 @@ class Event(Monitored, CommandManaged):
 	
 	#_____________________________________________________________________
 	
+	@UndoSystem.UndoCommand("AddAudioFadePoints", "temp", "temp2", "temp3", "temp4")
 	def RemoveAudioFadePoints(self, firstPoint, secondPoint, firstOldVolume=None, secondOldVolume=None):
 		"""
-		   Removed a point with values from the fade list.
-		   If firstOldVolume and secondOldVolume are given,
-		   the levels at the two points will be replaces with the
-		   old volumes instead of being removed.
-		    
-		   undo : AddAudioFadePoints: temp, temp2, temp3, temp4
+		Removes a fade point (along with its values) from the audioFadePoints list.
+		If firstOldVolume and secondOldVolume are given, the levels at the two points
+		will be replaced with those old values, instead of being removed.
+		
+		Parameters:
+			firstPoint -- point in time, where the fade starts.
+			secondPoint -- point in time, where the fade ends.
+			firstOldVolume -- old value of the initial fade volume,
+								used to replace the current one.
+			secondOldVolume -- old value of the final fade volume,
+								used to replace the current one.
 		"""
 		#undo values
 		self.temp = firstPoint
@@ -708,8 +931,12 @@ class Event(Monitored, CommandManaged):
 	
 	def __UpdateAudioFadePoints(self):
 		"""
-		   Private function that uses the private dictionary with 
-		   all the fade points to update the audioFadePoints list.
+		Private function that uses the private dictionary with
+		all the fade points to update the audioFadePoints list.
+		The audioFadePoints is different from the dictionary
+		because it will always have points at the beginning and
+		the end of the event (unless there are none at all), and
+		it is a list, so accessing the contents in order is much faster.
 		"""
 		
 		#update the fade points list from the dictionary
@@ -736,6 +963,12 @@ class Event(Monitored, CommandManaged):
 	#_____________________________________________________________________
 	
 	def __UpdateFadeLevels(self):
+		"""
+		Private function that uses the private dictionary with
+		all the fade leves to update the fadeLevels list. The fadeLevels
+		list is a cache of faded levels as they will be shown on the screen
+		so that we don't have to calculate them everytime we draw.
+		"""
 		if not self.audioFadePoints:
 			#there are no fade points for us to use
 			return
@@ -752,7 +985,7 @@ class Event(Monitored, CommandManaged):
 				fadePercents.extend([ fade[1] ] * levelsInThisSection)
 			else:
 				step = (fade[1] - previousFade[1]) / levelsInThisSection
-				floatList = floatRange(previousFade[1], fade[1], step)
+				floatList = Utils.floatRange(previousFade[1], fade[1], step)
 				#make sure the list of levels does not exceed the calculated length
 				floatList = floatList[:levelsInThisSection]
 				fadePercents.extend(floatList)
@@ -775,10 +1008,13 @@ class Event(Monitored, CommandManaged):
 	
 	def GetFadeLevels(self):
 		"""
-		   Returns a list of levels, the same length as the levels list.
-		   The only difference between this list and the levels list is
-		   that the levels in this list are scaled according to and fade
-		   curves applied to the current event.
+		Obtain the fade levels list.
+		The only difference between this list and the levels list is
+		that the levels in this list are scaled according to the fade
+		curves applied to the current Event.
+		
+		Returns:
+			a list of fade levels, the same length as the levels list.
 		"""
 		# no fades registered
 		if not self.audioFadePoints:
@@ -793,10 +1029,14 @@ class Event(Monitored, CommandManaged):
 	
 	def GetFadeLevelAtPoint(self, time):
 		"""
-		   Returns the level of the audio in percent (0-1)
-		   at the point given my time (in seconds)
-		"""
+		Obtain the level of audio at any point in time.
 		
+		Parameters:
+			time -- point in time to extract the audio level from.
+		
+		Returns:
+			the level of the audio in percentage format [0,1]
+		"""
 		if not self.audioFadePoints:
 			return 1.0
 		if self.audioFadePoints[-1][0] < time or self.audioFadePoints[0][0] > time:
@@ -824,6 +1064,9 @@ class Event(Monitored, CommandManaged):
 	#_____________________________________________________________________
 	
 	def DeleteSelectedFadePoints(self):
+		"""
+		Removes all the fade points for this Event.
+		"""
 		removeList = []
 		for key in self.__fadePointsDict.iterkeys():
 			if self.selection[0] <= key <= self.selection[1]:
