@@ -4,17 +4,17 @@
 #
 #	InstrumentViewer.py
 #	
-#	Encapsulate the customised track viewing and editing control
+#	Encapsulate the customised track viewing and editing control.
 #
 #-------------------------------------------------------------------------------
 
 import gtk
 import pango
-from Project import *
 from EventLaneViewer import *
 import Globals
 import InstrumentEffectsDialog
 import AddInstrumentDialog
+import Utils
 import gettext
 _ = gettext.gettext
 
@@ -22,18 +22,22 @@ _ = gettext.gettext
 
 class InstrumentViewer(gtk.EventBox):
 	""" 
-		Class to encapsulate the customised track viewing and editing control.
+	Encapsulates the customized track viewing and editing control.
 	"""
-
-	EDGE_DISTANCE = 5				# Size of event edge handle
-	EDGE_HOT_ZONE = 10				# Size of 'hot zone' used to trigger drawing of edge handle
-	BAR_WIDTH = 15
-	MAX_PEAK = 30
+	
+	""" Size of the instrument name label """
 	LABEL_WIDTH_CHARS = 12
+	
+	""" Widget color when unselected """
 	UNSELECTED_COLOUR = None
+	
+	""" Widget color when selected """
 	SELECTED_COLOUR = None
 	
-	INSTR_DRAG_TYPE = 83			# Number only to be used inside Jokosher
+	""" Number only to be used inside Jokosher """
+	INSTR_DRAG_TYPE = 83
+	
+	""" Custom numbers for use while dragging audio in Jokosher """
 	DRAG_TARGETS = [ ( "jokosher_instr_move", 	# A custom name for the instruments
 					   gtk.TARGET_SAME_APP,		# Only move inside Jo
 					   INSTR_DRAG_TYPE )]		# Use the custom number
@@ -43,12 +47,15 @@ class InstrumentViewer(gtk.EventBox):
 	def __init__(self, project, instrument, projectview, mainview, small = False):
 		gtk.EventBox.__init__(self)
 		"""
-			project - the current active project
-			instrument - the instrument that the event lane belongs
-			instrumentviewer - the instrumentviewer holding the event lane
-			projectview - the RecordingView instance that this belongs to
-			mainview - the main Jokosher window
-			small - set to True if we want small edit views (i.e. for mix view)
+		Creates a new instance of InstrumentViewer.
+		
+		Parameters:
+			project -- the currently active Project.
+			instrument -- the instrument that the event lane belongs.
+			instrumentviewer - the InstrumentViewer holding the event lane.
+			projectview - the RecordingView instance that this belongs to.
+			mainview - the MainApp Jokosher window.
+			small - set to True if we want small edit views (i.e. for mixing view).
 		"""
 		self.instrument = instrument
 		self.project = project
@@ -63,7 +70,6 @@ class InstrumentViewer(gtk.EventBox):
 		#get the default colour for the current theme
 		self.UNSELECTED_COLOR = self.get_style().bg[0]
 		
-		
 		self.mainBox = gtk.HBox()
 		self.add(self.mainBox)
 		
@@ -79,6 +85,7 @@ class InstrumentViewer(gtk.EventBox):
 
 		# create track header bits
 		self.labelbox = gtk.HBox()
+		self.labelbox.set_spacing(6)
 		self.labeleventbox = gtk.EventBox()
 		self.instrlabel = gtk.Label(self.instrument.name)
 		self.instrlabel.set_ellipsize(pango.ELLIPSIZE_END)
@@ -110,36 +117,41 @@ class InstrumentViewer(gtk.EventBox):
 			pb = self.instrument.pixbuf.scale_simple(20, 20, gtk.gdk.INTERP_BILINEAR)
 			self.image.set_from_pixbuf(pb)
 		
-		self.imageeventbox = gtk.EventBox()
-		self.imageeventbox.connect("button_release_event", self.OnChangeInstrumentType)
-		self.imageeventbox.add(self.image)
-		
-		self.labelbox.pack_start(self.imageeventbox, False)
+		self.labelbox.pack_start(self.image, False)
 		self.labelbox.pack_end(self.labeleventbox)
 		self.headerBox.pack_start(self.labelbox)
 		self.controlsBox = gtk.HBox()
 		self.headerBox.pack_start(self.controlsBox, False)
 		
+		# define the tooltip messages and images for buttons that change states
+		self.recTipDisabled = _("Enable this instrument for recording")
+		self.recTipEnabled = _("Disable this instrument for recording")
+		self.muteTipDisabled = _("Mute - silence this instrument")
+		self.muteTipEnabled = _("Unmute - hear this instrument")
+		self.soloTipDisabled = _("Activate Solo - silence all other instruments")
+		self.soloTipEnabled = _("Deactivate Solo - hear all the instruments")
+		
+		self.recImgDisabled = gtk.gdk.pixbuf_new_from_file(os.path.join(Globals.IMAGE_PATH, "icon_arm.png"))
+		self.recImgEnabled = gtk.gdk.pixbuf_new_from_file(os.path.join(Globals.IMAGE_PATH, "icon_disarm.png"))
+		self.soloImgDisabled = gtk.gdk.pixbuf_new_from_file(os.path.join(Globals.IMAGE_PATH, "icon_solo.png"))
+		self.soloImgEnabled = gtk.gdk.pixbuf_new_from_file(os.path.join(Globals.IMAGE_PATH, "icon_group.png"))
+		self.muteImgDisabled = Utils.GetIconThatMayBeMissing("stock_volume", gtk.ICON_SIZE_BUTTON, False)
+		self.muteImgEnabled = Utils.GetIconThatMayBeMissing("stock_volume-mute", gtk.ICON_SIZE_BUTTON, False)
+		
 		if not (self.small):
-			recimg = gtk.image_new_from_stock(gtk.STOCK_MEDIA_RECORD, gtk.ICON_SIZE_BUTTON)
 			self.recTip = gtk.Tooltips()
 			self.recButton = gtk.ToggleButton("")
-			self.recButton.set_property("image", recimg)
-			self.recTip.set_tip(self.recButton, _("Enable this instrument for recording"), None)
+			self.recTip.set_tip(self.recButton, self.recTipEnabled, None)
 			self.recButton.connect("toggled", self.OnArm)
 			
 			self.muteButton = gtk.ToggleButton("")
 			self.muteButton.connect("toggled", self.OnMute)
 			self.muteTip = gtk.Tooltips()
-			self.muteTip.set_tip(self.muteButton, _("Mute - silence this instrument"), None)
+			self.muteTip.set_tip(self.muteButton, self.muteTipDisabled, None)
 			
-			soloimg = gtk.Image()
-			soloimg.set_from_file(os.path.join(Globals.IMAGE_PATH, "icon_solo.png"))
 			self.soloButton = gtk.ToggleButton("")
-			self.soloButton.set_image(soloimg)
 			self.soloTip = gtk.Tooltips()
-			self.soloTip.set_tip(self.soloButton, _("Solo - silence all other instruments"), None)
-			#self.recButton.set_property("image", soloimg)
+			self.soloTip.set_tip(self.soloButton, self.soloTipDisabled, None)
 			self.soloButton.connect("toggled", self.OnSolo)
 			
 			self.propsButton = gtk.Button()
@@ -171,12 +183,18 @@ class InstrumentViewer(gtk.EventBox):
 		# Connect to drag_begin to add a custom icon
 		self.headerEventBox.connect('drag_begin', self.OnDragBegin)
 		self.headerEventBox.connect('drag_drop', self.OnDragDrop)
+		
+		self.Update()
 
 	#_____________________________________________________________________
 
 	def OnMute(self, widget):
 		"""
-			Callback for "toggle" signal on Mute button
+		Toggles muting the instrument on/off.
+		It will also update the pressed in/out look of the button.
+		
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly.
 		"""
 		if not self.Updating:
 			self.instrument.ToggleMuted(wasSolo=False)
@@ -185,7 +203,11 @@ class InstrumentViewer(gtk.EventBox):
 
 	def OnArm(self, widget):
 		"""
-			Callback for "toggle" signal on Record button
+		Toggles arming the instrument on/off.
+		It will also update the pressed in/out look of the button.
+		
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly.
 		"""
 		if not self.Updating:
 			self.instrument.ToggleArmed()
@@ -194,7 +216,8 @@ class InstrumentViewer(gtk.EventBox):
 	
 	def OnSolo(self, widget):
 		"""
-			Callback for "toggle" signal on Solo button
+		Toggles soloing the instrument on/off.
+		It will also update the pressed in/out look of the button.
 		"""
 		if not self.Updating:
 			self.instrument.ToggleSolo(False)
@@ -203,20 +226,41 @@ class InstrumentViewer(gtk.EventBox):
 
 	def OnSelect(self, widget, event):
 		"""
-			Callback for "button_press_event" anywhere within InstrumentViewer
-			Sets instrument to selected state
-		"""
+		Called when a button has been pressed anywhere within InstrumentViewer.
+		This method sets the instrument to a selected state.
 		
-		if 'GDK_CONTROL_MASK' not in event.state.value_names:
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly.
+			event -- reserved for GTK callbacks, don't use it explicitly.
+		
+		Returns:
+			True -- continue GTK signal propagation. *CHECK*
+		"""
+		if self.instrument.project.GetIsRecording():
+			return
+		
+		if 'GDK_CONTROL_MASK' in event.state.value_names:
+			self.instrument.SetSelected(True)
+		else:
 			self.project.ClearEventSelections()
 			self.project.SelectInstrument(self.instrument)
+		
 		return True
 
 	#_____________________________________________________________________
 
 	def OnEditLabel(self, widget, event):
 		"""
-			Callback for "button_press_event" in the instrument name label
+		Called when a button has been pressed within the instrument name label.
+		This method shows a text entry that allows the user to change the name
+		of the correspondent Instrument
+		
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly.
+			event -- reserved for GTK callbacks, don't use it explicitly.
+			
+		Returns:
+			True -- continue GTK signal propagation. *CHECK*
 		"""
 		if not self.instrument.isSelected:
 			self.OnSelect(widget, event)
@@ -246,7 +290,11 @@ class InstrumentViewer(gtk.EventBox):
 
 	def OnAcceptEditLabel(self, widget=None):
 		"""
-			Called when the instrument label has been edited
+		Called after the instrument label has been edited.
+		This method updates the instrument label with the label the user entered.
+		
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly.
 		"""
 		# change instrument name then replace edit label with normak label
 		if self.editlabelPacked:	
@@ -266,12 +314,22 @@ class InstrumentViewer(gtk.EventBox):
 		
 	#_____________________________________________________________________
 	
+	def Destroy(self):
+		"""
+		Called when the InstrumentViewer is closed
+		This method also destroys the corresponding EventLaneViewer.
+		"""
+		self.instrument.RemoveListener(self)
+		self.eventLane.Destroy()
+		self.destroy()
+	
+	#_____________________________________________________________________
+	
 	def Update(self):
 		"""
-			Called when requested by projectview.Update() to update
-			the display in response to a change in state in any object
-			it is listening to. In turn calls EventLaneViewer.Update()
-			for its EventLaneViewer.
+		Called when requested by projectview.Update() to update	the display,
+		in response to a change in state in any object is it listening to.
+		In turn calls EventLaneViewer.Update() for its EventLaneViewer.
 		"""
 		self.Updating = True
 
@@ -281,13 +339,39 @@ class InstrumentViewer(gtk.EventBox):
 			self.muteButton.set_active(self.instrument.actuallyIsMuted)
 			self.soloButton.set_active(self.instrument.isSolo)
 			self.soloTip.enable()
-		
+
+			# update the mute button image and tooltip
+			image = gtk.Image()
 			if self.instrument.actuallyIsMuted:
-				self.muteButton.set_image(gtk.image_new_from_icon_name("stock_volume-mute", gtk.ICON_SIZE_BUTTON))
-				self.muteTip.set_tip(self.muteButton, _("Muted"), None)
+				image.set_from_pixbuf(self.muteImgEnabled)
+				self.muteButton.set_image(image)
+				self.muteTip.set_tip(self.muteButton, self.muteTipEnabled, None)
 			else:
-				self.muteButton.set_image(gtk.image_new_from_icon_name("stock_volume", gtk.ICON_SIZE_BUTTON))
-				self.muteTip.set_tip(self.muteButton, _("Unmuted"), None)
+				image.set_from_pixbuf(self.muteImgDisabled)
+				self.muteButton.set_image(image)
+				self.muteTip.set_tip(self.muteButton, self.muteTipDisabled, None)
+			
+			# update the arm button image and tooltip	
+			image = gtk.Image()
+			if self.instrument.isArmed:
+				image.set_from_pixbuf(self.recImgEnabled)
+				self.recButton.set_image(image)
+				self.recTip.set_tip(self.recButton, self.recTipEnabled, None)
+			else:
+				image.set_from_pixbuf(self.recImgDisabled)
+				self.recButton.set_image(image)
+				self.recTip.set_tip(self.recButton, self.recTipDisabled, None)
+				
+			# update the solo button image and tooltip
+			image = gtk.Image()
+			if self.instrument.isSolo:
+				image.set_from_pixbuf(self.soloImgEnabled)
+				self.soloButton.set_image(image)
+				self.soloTip.set_tip(self.soloButton, self.soloTipEnabled, None)
+			else:
+				image.set_from_pixbuf(self.soloImgDisabled)
+				self.soloButton.set_image(image)
+				self.soloTip.set_tip(self.soloButton, self.soloTipDisabled, None)
 		
 		if self.instrument.isSelected:
 			#For some reason, putting self.style.base[3] in __init__ makes it return the wrong colour.
@@ -297,14 +381,12 @@ class InstrumentViewer(gtk.EventBox):
 			self.headerEventBox.modify_bg(gtk.STATE_NORMAL, self.SELECTED_COLOUR)
 			self.labeleventbox.modify_bg(gtk.STATE_NORMAL, self.SELECTED_COLOUR)
 			self.eventLane.modify_bg(gtk.STATE_NORMAL, self.SELECTED_COLOUR)
-			self.imageeventbox.modify_bg(gtk.STATE_NORMAL, self.SELECTED_COLOUR)
 			
 		else:
 			self.modify_bg(gtk.STATE_NORMAL, self.UNSELECTED_COLOUR)
 			self.headerEventBox.modify_bg(gtk.STATE_NORMAL, self.UNSELECTED_COLOUR)
 			self.labeleventbox.modify_bg(gtk.STATE_NORMAL, self.UNSELECTED_COLOUR)
 			self.eventLane.modify_bg(gtk.STATE_NORMAL, self.UNSELECTED_COLOUR)
-			self.imageeventbox.modify_bg(gtk.STATE_NORMAL, self.UNSELECTED_COLOUR)
 
 		self.instrlabel.set_text(self.instrument.name)
 		if self.editlabelPacked:
@@ -316,11 +398,16 @@ class InstrumentViewer(gtk.EventBox):
 
 	def OnMouseMove(self, widget, event):
 		"""
-			Callback for "enter_notify_event" and "leave_notify_event"
-			for the instrument name label. Changes cursor to show the 
-			text is edittable
+		Called when the mouse cursor enters or leaves the instrument name label area.
+		This method changes the cursor to show text is editable 
+		when hovered over the instrument label area.
+		
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly.
+			event -- reserved for GTK callbacks, don't use it explicitly.
 		"""
-		if not self.window: return
+		if not self.window or self.instrument.project.GetIsRecording():
+			return
 		if (event.type == gtk.gdk.ENTER_NOTIFY):
 			self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.XTERM))
 		else:
@@ -330,9 +417,14 @@ class InstrumentViewer(gtk.EventBox):
 
 	def ResizeHeader(self, width):
 		"""
-			Changes the padding space of the header box in order to line
-			up correctly with the timeline display. 
-			NOTE: Not called here but from TimeLineBar.py
+		Changes the padding space of the header box in order to line
+		up correctly with the timeline display.
+		
+		Considerations:
+			This method should be called from TimeLineBar.
+			
+		Parameters:
+			width -- new size of the header box.
 		"""
 		padding = width - self.headerBox.size_request()[0]
 		self.headerAlign.set_padding(0, 0, 0, padding)
@@ -341,28 +433,53 @@ class InstrumentViewer(gtk.EventBox):
 	#______________________________________________________________________
 
 	def OnInstrumentEffects(self, widget, mouse):
-		""" Creates and shows the instrument effects dialog"""
+		"""
+		Creates and shows the instrument effects dialog
+		
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly.
+			mouse -- reserved for GTK callbacks, don't use it explicitly.
+		"""
 		Globals.debug("props button pressed")
 		if not self.effectsDialog:
-			self.effectsDialog = InstrumentEffectsDialog.InstrumentEffectsDialog(self.instrument, self.OnInstrumentEffectsDestroyed)
+			self.effectsDialog = InstrumentEffectsDialog.InstrumentEffectsDialog(
+					self.instrument,
+					self.OnInstrumentEffectsDestroyed,
+					self.mainview.icon)
 		else:
 			self.effectsDialog.BringWindowToFront()
 
 	#______________________________________________________________________
 	
 	def OnInstrumentEffectsDestroyed(self, window):
+		"""
+		Called when the InstrumentEffectsDialog is destroyed.
+		
+		Parameters:
+			window -- reserved for GTK callbacks, don't use it explicitly.
+		"""
 		self.effectsDialog = None
 		
 	#______________________________________________________________________
 	
 	def OnDragMotion(self, widget, context, x, y, time):
-		'''
-			Called each time the user moves his/her mouse while dragging.
-			'if' clause checks if mouse is on an instrument that isn't the
-			source instrument. If so, it swaps that instrument and the
-			source instrument in the GUI. Swapping of the Instrument objects
-			in self.project.instruments happens in OnDragDrop().
-		'''
+		"""
+		Called each time the user moves the mouse while dragging.
+		If the mouse is on an instrument that isn't the source
+		instrument, it swaps that instrument and the source instrument
+		in the GUI. Swapping of the Instrument objects in self.project.instruments
+		happens in OnDragDrop().
+		
+		Parameters:
+			widget -- InstrumentViewer the mouse is hovering over.
+			context -- cairo widget context. Used to extract the source instrument.
+			x -- reserved for GTK callbacks, don't use it explicitly.
+			y -- reserved for GTK callbacks, don't use it explicitly.
+			time -- reserved for GTK callbacks, don't use it explicitly.
+		
+		Returns:
+			True -- continue GTK signal propagation. *CHECK*
+		"""
 		source = context.get_source_widget() 	# Will return an EventBox (self.headerEventBox)
 		if widget != source:					# Dont swap with self
 			box = self.GetInstrumentViewVBox()
@@ -380,23 +497,36 @@ class InstrumentViewer(gtk.EventBox):
 	#______________________________________________________________________
 	
 	def OnDragBegin(self, widget, context):
-		'''
-			Called at the start of the drag and drop.
-			Displays the instrument icon when dragging.
-		'''
+		"""
+		Called when the drag and drop procedure begins.
+		This method will display the instrument icon when dragging.
+		
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly.
+			context -- a cairo context widget.
+		"""
 		widget.drag_source_set_icon_pixbuf(self.instrument.pixbuf)
 		return True
 	
 	#______________________________________________________________________
 	
 	def OnDragDrop(self, widget, context, x, y, time):
-		'''
-			Called when the user releases MOUSE1.
-			Calls MoveInstrument, which moves the dragged
-			instrument to the end position in the
-			self.project.instruments array.
-			MoveInstrument is undo-able.
-		'''
+		"""
+		Called when the user releases MOUSE1, finishing a drag and drop
+		procedure.
+		Calls MoveInstrument, which moves the dragged instrument to
+		the end position in the self.project.instruments array.
+		
+		Considerations:
+			the MoveInstrument action is undo-able.
+			
+		Parameters:
+			widget -- InstrumentViewer being dragged.
+			context -- reserved for GTK callbacks, don't use it explicitly.
+			x -- reserved for GTK callbacks, don't use it explicitly.
+			y -- reserved for GTK callbacks, don't use it explicitly.
+			time -- reserved for GTK callbacks, don't use it explicitly.
+		"""
 		id = self.instrument.id
 		box = self.GetInstrumentViewVBox()
 		position = box.get_children().index(self)
@@ -411,10 +541,13 @@ class InstrumentViewer(gtk.EventBox):
 	#______________________________________________________________________
 
 	def GetInstrumentViewVBox(self):
-		'''
-			Returns the instrumentBox if the current view is a RecordingView.
-			Returns the timebox if the current view is a CompactMixView.
-		'''
+		"""
+		Obtain the current Instrument view box.
+		
+		Returns:
+			the instrumentBox if the current view is a RecordingView.
+			the timebox if the current view is a CompactMixView.
+		"""
 		if hasattr(self.projectview, "instrumentBox"):
 			return self.projectview.instrumentBox
 		else:
@@ -422,8 +555,17 @@ class InstrumentViewer(gtk.EventBox):
 	
 	#_____________________________________________________________________
 	
-	def OnStateChanged(self, obj, change=None):
-		if change == self.instrument.IMAGE:
+	def OnStateChanged(self, obj, change=None, *extra):
+		"""
+		Called when a change of state is signalled by any of the
+		objects this view is 'listening' to.
+		
+		Parameters:
+			obj -- object changing state. *CHECK*
+			change -- the change which has occured.
+			extra -- extra parameters passed by the caller.
+		"""
+		if change == "image":
 			self.image.clear()
 			self.image.set_from_pixbuf(self.instrument.pixbuf)
 	
@@ -431,16 +573,26 @@ class InstrumentViewer(gtk.EventBox):
 
 	def OnChangeInstrumentType(self, widget, event):
 		"""
-			Callback for "button_press_event" in the instrument header icon
+		Called when a button has been pressed in the instrument header icon.
+		This method displays the AddInstrumentDialog, allowing the user
+		to change the selected instrument.
+		
+		Parameters:
+			widget -- reserved for GTK callbacks, don't use it explicitly.
+			event -- reserved for GTK callbacks, don't use it explicitly.
+			
+		Returns:
+			True -- if there's no instrument selected, select one and then
+					continue GTK signal propagation. *CHECK*
 		"""
 		if not self.instrument.isSelected:
 			self.OnSelect(widget, event)
 			# Don't edit type unless the user clicks while we are already selected
 			return True
 		
-		AddInstrumentDialog.AddInstrumentDialog(self.project, self.mainview, 
-		    self.instrument)
+		AddInstrumentDialog.AddInstrumentDialog(self.project, self.mainview, self.instrument)
 		    
 	#______________________________________________________________________
 
+
 	#=========================================================================	
